import java.io.*;

// Demonstrate the power of custom serialization
class Node implements Externalizable {
	public int value;
	
	// Create a cyclic graph of nodes. Note that the object reference
	// cache will prevent the output stream to stuck in an infinite loop
	// through the serialization of the object graph.
	public Node parent;
	
	public Node left;
	public Node right;
	
	// Externalizable classes must have a no-arg constructor
	public Node() {}
	
	public Node(int v, Node p) {
		value = v;
		parent = p;
	}
	
	@Override
	public void writeExternal(ObjectOutput out)
		throws IOException {
		// Write custom serialization here, or use the default method:
		// out.defaultWriteObject();
		out.writeObject(parent);
		out.writeObject(left);
		out.writeObject(right);
		out.writeInt(value);
	}
	
	@Override
	public void readExternal(ObjectInput in)
		throws IOException, ClassNotFoundException {
		// Write custom deserialization here, or use the default method:
		// in.defaultReadObject();
		parent = (Node)in.readObject();
		left = (Node)in.readObject();
		right = (Node)in.readObject();
		value = in.readInt();
		
		// On-the-fly calculated values can also be restored here.
	}
	
	@Override
	public String toString() {
		return "Node #" + value + " { " + 
			(left != null ? left.toString() : "NULL") + ", " +
			(right != null ? right.toString() : "NULL") + " }";
	}
}
