
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    private static final int PORT = 1234;

    public static void main(String[] args) {
        try {
            // Open listener socket
            ServerSocket server = new ServerSocket(PORT);

            // Indicate startup
            System.out.println("Server started to listen on port " + PORT);

            // Run the service continuously
            while(true) {
                // Look for a client connection, accept when found one
                Socket client = server.accept();

                // Process it on a separate thread
                ServerThread st = new ServerThread(client);
                st.start();
            }
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static class ServerThread extends Thread {

        private Socket client;

        public ServerThread(Socket client) {
            this.client = client;
        }

        @Override
        public void run() {
            try {
                // Receive request
                InputStream is = client.getInputStream();
                ObjectInputStream ois = new ObjectInputStream(is);
                String request = (String)ois.readObject();

                // Process request
                System.out.println("Request: " + request);

                // Create response
                // Note that the response is fixed now,
                // but usually we use the request to construct it
                String response = "Another test message, from the server.";

                // Send response
                OutputStream os = client.getOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(os);
                oos.writeObject(response);
                oos.flush(); // important: immediately write contents to the socket
            } catch (IOException | ClassNotFoundException ex) {
                Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                // Close connection to the client
                try {
                    client.close();
                } catch (IOException ex) {
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

    }
}
