
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ServerMain {

    private static final int PORT = 1234;

    public static void main(String[] args) {
        try {
            // Open listener socket
            ServerSocket server = new ServerSocket(PORT);

            // Indicate startup
            System.out.println("Server started to listen on port " + PORT);

            // Run the service continuously
            while(true) {
                // Look for a client connection, accept when found one
                Socket client = server.accept();

                // Process it on a separate thread
                ServerThread st = new ServerThread(client);
                st.start();
            }
        } catch (IOException ex) {
            Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static class ServerThread extends Thread {

        private Socket client;

        public ServerThread(Socket client) {
            this.client = client;
        }

        @Override
        public void run() {
            try {
                // Receive request: read out the operands
                InputStream is = client.getInputStream();
                ObjectInputStream ois = new ObjectInputStream(is);
                int a = ois.readInt();
                int b = ois.readInt();

                // Process request and create response:
                // invoke the service implementation
                CalculatorServiceImpl impl = new CalculatorServiceImpl();
                int result = impl.add(a, b);

                // Send response: write in the result of the invocation
                OutputStream os = client.getOutputStream();
                ObjectOutputStream oos = new ObjectOutputStream(os);
                oos.writeInt(result);
                oos.flush(); // important: immediately write contents to the socket
            } catch (IOException ex) {
                Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
            } finally {
                // Close connection to the client
                try {
                    client.close();
                } catch (IOException ex) {
                    Logger.getLogger(ServerMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

    }
}
