
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ClientMain {

    private static final String HOST = "localhost";
    private static final int PORT = 1234;

    public static void main(String[] args) {
        System.out.println("Addition result: " + callAdd(10, 20));
        System.out.println("Square root result: " + callSqrt(2));
    }

    private static int callAdd(int a, int b) {
        Object[] params = new Object[2];
        params[0] = a;
        params[1] = b;
        return (int)call("add", params);
    }

    private static double callSqrt(double a) {
        Object[] params = new Object[1];
        params[0] = a;
        return (double)call("sqrt", params);
    }

    private static Object call(String methodName, Object[] params) {
        Socket s = null;

        try {
            // Open connection to the server
            s = new Socket(HOST, PORT);

            // Send request: write in the method name and the operands
            OutputStream os = s.getOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(os);
            oos.writeUTF(methodName);
            oos.writeInt(params.length);

            // Serialize the value and type of every parameter
            for(Object p : params) {
                oos.writeObject(p);
                oos.writeObject(p.getClass());
            }

            oos.flush(); // important: immediately write contents to the socket

            // Receive response: read out the addition result
            InputStream is = s.getInputStream();
            ObjectInputStream ois = new ObjectInputStream(is);
            Object response = ois.readObject();

            // Return response
            return response;
        } catch (UnknownHostException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException | ClassNotFoundException ex) {
            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            // Close connection to the server
            if(s != null) {
                try {
                    s.close();
                } catch (IOException ex) {
                    Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        // Fall-back value in the case of an exception
        return null;
    }
}
