
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;


// This class helps to hide the concrete invocation method.
public class CalculatorServiceLocator {

    private static final String HOST = "localhost";
    private static final int PORT = 1234;

    public static CalculatorService getService() {
        // Assume that this is a configuration variable
        boolean useRemoteImpl = true;

        if(useRemoteImpl) {
            return getRemoteImplementation();
        } else {
            return getLocalImplementation();
        }
    }

    // Create a dynamic proxy to mimc server interface
    private static CalculatorService getRemoteImplementation() {
        return (CalculatorService)Proxy.newProxyInstance(
            CalculatorServiceLocator.class.getClassLoader(),
            new Class<?>[] { CalculatorService.class },
            new InvocationHandler() {

            @Override
            public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
                Socket s = null;

                try {
                    // Open connection to the server
                    s = new Socket(HOST, PORT);

                    // Send request: write in the method name and the operands
                    OutputStream os = s.getOutputStream();
                    ObjectOutputStream oos = new ObjectOutputStream(os);
                    oos.writeUTF(method.getName());
                    oos.writeInt(args.length);

                    // Serialize the value and type of every parameter
                    for(Object p : args) {
                        oos.writeObject(p);
                        oos.writeObject(p.getClass());
                    }

                    oos.flush(); // important: immediately write contents to the socket

                    // Receive response: read out the addition result
                    InputStream is = s.getInputStream();
                    ObjectInputStream ois = new ObjectInputStream(is);
                    InvocationResult response = (InvocationResult)ois.readObject();

                    // Process response
                    if(response.isException()) {
                        throw response.getException();
                    }

                    return response.getResult();
                } catch (UnknownHostException ex) {
                    Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                } catch (IOException | ClassNotFoundException ex) {
                    Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                } finally {
                    // Close connection to the server
                    if(s != null) {
                        try {
                            s.close();
                        } catch (IOException ex) {
                            Logger.getLogger(ClientMain.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                }

                // Fall-back value in the case of an exception
                return null;
            }
        });
    }

    // This method returns an imprecise, local implementation of the service.
    private static CalculatorService getLocalImplementation() {
        return new CalculatorService() {

            @Override
            public Integer add(Integer a, Integer b) {
                return a + b + 1;
            }

            @Override
            public Double sqrt(Double a) {
                return Math.sqrt(a) + 1;
            }
        };
    }
}
