
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.derby.jdbc.EmbeddedDataSource;

public class Main {

    // Set it to false to use the implementation which prevents SQL injection
    private static final boolean USE_VULNERABLE_IMPLEMENTATION = true;

    public static void main(String[] args) {
        // Create an embedded Derby data source
        EmbeddedDataSource eds = new EmbeddedDataSource();
        eds.setDatabaseName("testdb");
        eds.setCreateDatabase("create");

        try (Connection connection = eds.getConnection()) {
            // Prepare the database (reset if any argument provided)
            prepareDatabase(connection, args.length > 0);

            // Simulate a command line user login
            simulateLogin(connection);
        } catch (SQLException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private static void prepareDatabase(Connection connection, boolean reset) throws SQLException {
        try {
            if (reset) {
                // Reset the database by dropping the table
                connection.createStatement().execute("DROP TABLE app_users;");
            }

            // Start a transaction
            connection.setAutoCommit(false);

            // Check that our table exists
            DatabaseMetaData metaData = connection.getMetaData();

            // Filter only by table name (note the upper case), and type
            ResultSet tables = metaData.getTables(null, null, "APP_USERS", new String[] { "TABLE" });

            // Check that the result set is empty
            if(!tables.next()) {
                // Create a statement that will create the table
                try (Statement stmt = connection.createStatement()) {
                    stmt.execute(
                        "CREATE TABLE app_users (" +
                            "name VARCHAR(255)," +
                            "password VARCHAR(255)," +
                            "level INT" +
                        ")"
                    );
                }

                // Fill the table with users
                // !!!
                // Note that storing passwords in plain text is very unsafe!
                // Use a strong, salted password hash function, eg. bcrypt!
                // !!!
                try (Statement stmt = connection.createStatement()) {
                    stmt.execute("INSERT INTO app_users VALUES('A', 'alpha', 1)");
                    stmt.execute("INSERT INTO app_users VALUES('B', 'beta', 2)");
                    stmt.execute("INSERT INTO app_users VALUES('C', 'charlie', 3)");
                }
            }

            tables.close();

            // Commit the transaction
            connection.commit();
        } catch (SQLException ex) {
            try {
                // Invalidate transaction
                connection.rollback();
            } catch (SQLException e) {
                throw e;
            }

            throw ex;
        } finally {
            // Go back to non-transactional mode
            connection.setAutoCommit(true);
        }
    }

    private static void simulateLogin(Connection connection) throws SQLException {
        // Get input from the console
        System.out.println("Please log in!");

        System.out.print("User name: ");
        String userName = System.console().readLine();

        System.out.print("Password: ");
        String password = System.console().readLine();

        // Initialize the statement and the result
        Statement stmt = null;
        ResultSet result = null;

        if (USE_VULNERABLE_IMPLEMENTATION) {
            // Assemble the login query
            String query = "SELECT name, level FROM app_users WHERE " +
                           "name = '" + userName +
                           "' AND password='" + password + "'";
            
            // Print out the query for debugging purposes
            System.out.println();
            System.out.println("DEBUG\t------------------------");
            System.out.println("DEBUG\tThe assembled query is: ");
            System.out.println("DEBUG\t" + query);
            System.out.println("DEBUG\t------------------------");
            System.out.println();
            
            stmt = connection.createStatement();
            result = stmt.executeQuery(query);
        } else {
            PreparedStatement pstmt = connection.prepareStatement(
                "SELECT name, level FROM app_users WHERE " +
                "name = ? AND password = ?");
            pstmt.setString(1, userName);
            pstmt.setString(2, password);
            result = pstmt.executeQuery();
            
            // Allows to close it later
            stmt = pstmt;
        }
        
        // Print out the login result
        if (!result.next()) {
            System.out.println("Invalid credentials!");

            // Try to login again
            simulateLogin(connection);
        } else {
            System.out.println("Login successful!");
            System.out.println("User name: " + result.getString(1) + ", level: " + result.getInt(2));
        }

        stmt.close();
    }
}
