
import entities.Product;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.TypedQuery;

public class Main {

    public static void main(String[] args) {
        // Create an entity manager and begin a transaction
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("testpu");
        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();

        // Insert some products
        for(int i = 0; i < 10; ++i) {
            Product p = new Product();
            p.setName("product" + i);
            p.setType(Product.Type.B);
            p.setLastModified(new Date());
            em.persist(p);
        }

        // Find a product by its primary key and update it
        Product p3 = em.find(Product.class, 3L);
        p3.setType(Product.Type.A);
        em.persist(p3);

        // Query products with a substituted (and named) filter parameter
        TypedQuery<Product> tq = em.createQuery(
            "select p from Product p where p.type = :prodType order by p.name",
            Product.class);
        tq.setParameter("prodType", Product.Type.B);
        printResults(tq.getResultList());

        // Use a named query defined on the entity
        TypedQuery<Product> nq = em.createNamedQuery("getAllProducts", Product.class);
        printResults(nq.getResultList());


        // Apply the changes before we clear the entity manager below
        em.flush();
        // Select an entity by its primary key
        Product p7A = em.find(Product.class, 7L);
        // Empty the cache
        em.clear();
        // Select the same entity again (now it's truly a different Java object)
        Product p7B = em.find(Product.class, 7L);

        // Test entity equivalence        
        System.out.println(p7A == p7B); // should be false
        System.out.println(p7A.equals(p7B)); // must be true
        
        // Commit the transaction and close the entity manager
        em.getTransaction().commit();
        em.close();
        emf.close();
    }

    private static void printResults(List<Product> results) {
        System.out.println();
        
        for(Product p : results) {
            System.out.println(p.toString());
        }
        
        System.out.println();
    }
}
