
package entities;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

// This indicates for JPA that this is an entity class
@Entity
// Rename the table - its also possible to specify schema, catalog and indicies
@Table(name = "pr")
// Add a named query, which can be referenced by name later
@NamedQueries({
    @NamedQuery(name = "getAllProducts",
        query = "select p from Product p order by p.id desc")
})
// The Serializable interface is not really necessary for entity classes
public class Product /* implements Serializable */ {
    /* private static final long serialVersionUID = 1L; */

    // Product type (just an example of a nested enumeration)
    public enum Type { A, B, C };

    // Indicates that this field is the primary key
    @Id
    // Its value will be automatically generated by the database
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    // Rename column, and specify the maximum storage length of the string
    @Column(name = "pn", length = 100)
    private String name;

    // Save only the time part of the embedded date instance
    @Temporal(TemporalType.TIME)
    private Date lastModified;

    // Use the string representation of enumeration values on storage
    @Enumerated(EnumType.STRING)
    private Type type;
    
    // Visible to Java serialization, but not persisted by JPA
    @Transient
    private int calculatedField;

    // Unused by both JPA persistence and Java serialization
    transient Thread saveThread;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Date getLastModified() {
        return lastModified;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    // The next two method implementations refelect the idea that all entities
    // with the same primary key considered as equal
    @Override
    public boolean equals(Object object) {
        // Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Product)) {
            return false;
        }
        Product other = (Product) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return
            "eak.Product[ id = " + id + ", name = " + name +
            ", lastModified = " + lastModified + ", type = " + type + " ]";
    }

}
