package hu.elte.eak;

import java.io.*;
import java.lang.reflect.*;

// Important note: the correct exception handling is omitted
// through all of these examples of serialization.
class App {
	
	private static final String TEST_SUBJECT_CLASSNAME = "hu.elte.eak.TestSubject";
	
	public static void main(String[] args)
		throws Exception {
		Class<?> tsk = testClassLoadingByName();
		testInvokingPrivateMethod(tsk);
		testInspectingAnnotationData(tsk);
	}
	
	private static Class<?> testClassLoadingByName()
		throws ClassNotFoundException {
		// return Class.forName(TEST_SUBJECT_CLASSNAME);
		ClassLoader loader = App.class.getClassLoader();
		return loader.loadClass(TEST_SUBJECT_CLASSNAME);
	}
	
	private static void testInvokingPrivateMethod(Class<?> tsk)
		throws NoSuchMethodException, IllegalAccessException,
			InstantiationException, InvocationTargetException {
		// Create a test instance
		Object tsi = tsk.newInstance();
		// TestSubject tsi = new TestSubject();
		
		// Get the class definition, the root of reflection informations
		// Class<TestSubject> tsk = TestSubject.class;
		// Class<TestSubject> tsk = tsi.getClass();
		
		// Get the `privateMethod` method
		Method pm = tsk.getDeclaredMethod("privateMethod");
		
		// Set it accessible. Note that it throws a SecurityException
		// if the access is denied by the current security manager.
		pm.setAccessible(true);
		
		// Invoke it
		pm.invoke(tsi);
	}
	
	private static void testInspectingAnnotationData(Class<?> tsk)
		throws NoSuchFieldException {
		Field d = tsk.getDeclaredField("data");
		CustomAnnotation ca = d.getAnnotation(CustomAnnotation.class);
		System.out.println("CustomAnnotation { id = " + ca.id()
			+ ", isImportant = " + ca.isImportant() + " }");
	}
}
