package eak;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebInitParam;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// Registers this class as a servlet.
@WebServlet(
	name = "hello-servlet",
	// Specifies the patterns for the URLs where this servlet will be accessible. 
	urlPatterns = { "/hello" },
	// Assign default value to the initialization parameters.
	initParams = {
		@WebInitParam(name = "color", value = "red")
	})
public class HelloServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	// Stores the value of an initialization parameter.
	private String color;
	
	private String servletName;
	
	public void init(ServletConfig config) throws ServletException {
		// Called by the servlet container to indicate to a servlet
		// that the servlet is being placed into service. 
		
		// Get the value of an initialization parameter.
		color = config.getInitParameter("color");
		
		servletName = config.getServletName();
	}

	public void destroy() {
		// Called by the servlet container to indicate to a servlet
		// that the servlet is being taken out of service.
	}

	// This method will be called by the service(...) method when the request method is the HTTP GET.
	// Note that multiple server threads can call this method at the same time, so it must be thread-safe.
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("text/html");
		response.setCharacterEncoding("utf-8");
		
		// This is equal with the two above calls.
		// response.addHeader("Content-Type", "text/html; charset=utf-8");
		
		// To write binary data to the response body, use this stream:
		// ServletOutputStream out = response.getOutputStream();
		// out.write(...);
		
		// To write character data to the response body, use this print writer:
		PrintWriter pw = response.getWriter();
		pw.println("<!DOCTYPE html>");
		pw.println("<html>");
		pw.println("<head>");
		pw.println("<meta charset=\"UTF-8\">");
		pw.println("<title>Servlet test page</title>");
		pw.println("</head>");
		pw.println("<body>");
		pw.println("<h1>This page is generated by " + servletName + ".</h1>");
		pw.println("The value of the initialization parameter is: ");
		pw.println("<em style=\"color: " + color + ";\">" + color + "</em>");
		pw.println("</body>");
		pw.println("</html>");
		
		// Sends the contents of the response body buffer to the network.
		// It is not possible to modify or add headers after this call.
		response.flushBuffer();
		response.addHeader("X-Flush-Indicator", "this-will-not-be-sent");
		
		// Set the response status. This case it is not needed (sets the default).
		response.setStatus(HttpServletResponse.SC_OK);
		
		// To activate the built-in error page mechanism, send error codes this way:
		// response.sendError(HttpServletResponse.SC_NOT_FOUND);
	}
}
