package eak;

import java.io.IOException;
import java.io.PrintWriter;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.TypedQuery;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.UserTransaction;

@WebServlet("/")
public class BookListing extends HttpServlet {
	private static final long serialVersionUID = 1L;

	// Inject an entity manager factory. Do not inject an entity manager with
	// @PersistenceContext directly, because they are not thread safe!
	@PersistenceUnit(unitName = "books")
	private EntityManagerFactory emf;

	// Retrieve a container managed transaction instance
	@Resource
	private UserTransaction utx;

	@Override
	public void init() throws ServletException {
		try {
			// Persist some entities in a transaction
			utx.begin();
			EntityManager em = emf.createEntityManager();
			for (int i = 0; i < 10; ++i) {
				Book b = new Book(i, "Book #" + i);
				em.persist(b);
			}
			utx.commit();
		} catch (Exception e) {
			// The container will not start using this service
			// when the initialization fails
			throw new ServletException(e);
		}
	}

	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// Prepare response headers
		response.setContentType("text/html");
		response.setCharacterEncoding("utf-8");

		PrintWriter pw = response.getWriter();
		pw.println("<!DOCTYPE html>");
		pw.println("<html>");
		pw.println("<head>");
		pw.println("<meta charset=\"UTF-8\">");
		pw.println("<title>Servlet test page</title>");
		pw.println("</head>");
		pw.println("<body>");
		pw.println("<h1>Books</h1>");
		pw.println("<table>");
		pw.println("<tr><th>Id</th><th>Title</th></tr>");

		// Use a separate entity manager instance each time. The factory itself
		// is thread-safe, but the individual instances are not!
		EntityManager em = emf.createEntityManager();
		// We made no modifications, so using a transaction here is optional
		TypedQuery<Book> query = em.createQuery("SELECT b FROM Book b",
				Book.class);
		for (Book b : query.getResultList()) {
			pw.println(String.format("<tr><td>%s</td><td>%s</td></tr>",
					b.getId(), b.getTitle()));
		}
		// Close entity managers properly to release their resources
		em.close();

		pw.println("</table>");
		pw.println("</body>");
		pw.println("</html>");
		pw.close();
	}

}
