package eak;

// A simple JavaBean (and not an EJB!) to store game data
public class GameBean {

	public static int SOLUTION_RANGE_MIN = 1;
	public static int SOLUTION_RANGE_MAX = 10;
	
	// The solution of the game
	private int solution;
	
	// How many times the user tried to guess it
	private int numberOfGuesses;
	
	// State indicator flags
	private boolean invalidNumber;
	private boolean incorrectGuess;
	private boolean foundTheSolution;
	
	// Property getters and a simple constructor
	public int getSolution() {
		return solution;
	}

	public int getNumberOfGuesses() {
		return numberOfGuesses;
	}

	public boolean isInvalidNumber() {
		return invalidNumber;
	}

	public boolean isIncorrectGuess() {
		return incorrectGuess;
	}

	public boolean isFoundTheSolution() {
		return foundTheSolution;
	}
	
	public GameBean() {
		reset();
	}
	
	// Generate a new solution and reset counters and flags 
	public void reset() {
		int rangeLength = SOLUTION_RANGE_MAX - SOLUTION_RANGE_MIN + 1;
		solution = (int)Math.floor(Math.random() * rangeLength + SOLUTION_RANGE_MIN);
		numberOfGuesses = 0;
		invalidNumber = false;
		incorrectGuess = false;
		foundTheSolution = false;
	}

	// Returns whether the given input is valid 
	public boolean checkSolution(String formInput) {
		invalidNumber = false;
		incorrectGuess = false;
		int currentGuess = 0;
		
		try {
			currentGuess = Integer.parseInt(formInput);
		} catch(NumberFormatException ex) {
			invalidNumber = true;
		}
		
		invalidNumber = invalidNumber
			|| currentGuess < SOLUTION_RANGE_MIN
			|| SOLUTION_RANGE_MAX < currentGuess; 
		
		numberOfGuesses += invalidNumber ? 0 : 1;
		foundTheSolution = !invalidNumber && currentGuess == solution;
		incorrectGuess = !invalidNumber && !foundTheSolution;
		
		return foundTheSolution;
	}
	
	// Simulating "flash scope" form some error indicator flags
	public void clearFlags() {
		invalidNumber = false;
		incorrectGuess = false;
		// The "foundTheSolution" flag does not need to be cleared,
		// because then the whole bean will be reset.
	}
}
