import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import eak.Calculator;
import eak.ValueHolder;


// Running as a standalone client:
// java -cp .:<glassfish-top-dir>/glassfish/lib/gf-client.jar:calculator-client-jndi.jar Main

// Running in an application client container:
// <glassfish-top-dir>/glassfish/bin/appclient -cp .:calculator-client-jndi.jar  Main

// !!! Important: the calculator-interface.jar must be in the same directory as the client!
public class Main {
		
	public static void main(String[] args) {
		Calculator calculator;
		Context ctx = null;
		
		try {
			// Explicitly specify the location of the server
			// (these are the default values, so this block is unnecessary)
			Properties environment = new Properties();
			environment.setProperty("org.omg.CORBA.ORBInitialHost", "127.0.0.1");
			environment.setProperty("org.omg.CORBA.ORBInitialPort", "3700");
			
			// Find the EJB with a JNDI lookup
			ctx = new InitialContext(environment);
			calculator = (Calculator)ctx.lookup(
				// This is a portable JNDI name of the bean. Format:
				// java:global/[<application-name>]/<module-name>/<bean-name>!<fully-qualified-bean-interface-name>
				// For more information on portable global JNDI names see:
				// https://blogs.oracle.com/MaheshKannan/entry/portable_global_jndi_names
				"java:global/calculator-application/calculator-ejb/calcBean!eak.Calculator"
			);
		} catch(NamingException ex) {
			ex.printStackTrace();
			return;
		} finally {
			if (ctx != null) {
				try {
					ctx.close();
				} catch (NamingException e) {
					e.printStackTrace();
					return;
				}
			}
		}
		
		// Call bean methods
		ValueHolder result;
		
		result = calculator.add(new ValueHolder(41), new ValueHolder(1));
		System.out.println("The sum of 41 and 1 is " + result.value);
		
		// Changing the value in the result will not have any effect in the
		// server, because it is passed by-copy.
		result.value = 2;
		
		// It is not sure this method will be called on the same
		// bean instance in the server as the previous.
		result = calculator.sqrt(result);
		System.out.println("The square root of 2 is " + result.value);
	}
}
