package eak;

import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.interceptor.Interceptors;

// Define a stateful session bean with a remote interface.
@Stateful
@Remote(StatefulCalculator.class)
// Setup the interceptors for this class
@Interceptors({
	Logger.class
	// When enabled, it changes the calculation result!
	//, RandomResultGenerator.class
})
public class CalculatorBean implements StatefulCalculator {

	// Inject a reference to the singleton EJB
	@EJB
	private ResultPersistence rp;
	
	// This is the internal state of the session bean.
	// It is guaranteed by the container that only one client will access
	// its value on a concrete bean instance.
    int total = 0;
    
    // Implementation of business methods
    
	@Override
	public void add(int amount) {
		total += amount;
	}

	@Override
	public void sub(int amount) {
		total -= amount;
	}
	
	@Override
	public void sqrt() throws CalculatorException {
		if (total < 0) {
			throw new CalculatorException("Cannot calculate the square root of a negative number!");
		}
		total = (int)Math.round(Math.sqrt(total));
	}

	// When the client calls a method annotated with @Remove, the container
	// will drop the bean instance after the call. Any further calls through
	// the same bean reference will throw an exception.
	@Remove
	@Override
	public int getResult() {
		rp.persistWithTimestamp(total);
		return total;
	}

}
