package eak.tx;

import java.util.List;

import javax.ejb.EJBException;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import eak.tx.entities.Message;
import eak.tx.exceptions.FatalMessageException;
import eak.tx.exceptions.MessageException;
import eak.tx.interceptors.InvocationLogger;

@Stateless
// Let the container manage the transactions. This is the default behavior.
@TransactionManagement(TransactionManagementType.CONTAINER)
// Log all invocations
@Interceptors({ InvocationLogger.class })
public class StatelessMessagesBean implements Messages {

	@PersistenceContext
	private EntityManager em;

	@Override
	// Ensure it always runs with an existing, or with a new transaction.
	// This is also the default value.
	@TransactionAttribute(TransactionAttributeType.REQUIRED)
	public void add(String text, ErrorType errorType) throws MessageException {
		// Try to persist the message
		em.persist(new Message(text));

		// Throw the specified exception
		switch (errorType) {
		case Runtime:
			throw new RuntimeException("Runtime exception thrown.");
		case Sys:
			throw new EJBException("System exception thrown.");
		case AppNonRollback:
			throw new MessageException("Non-rollback app exception thrown.");
		case AppRollback:
			throw new FatalMessageException("Rollback app exception thrown.");
		default:
			// Intentionally left blank, do nothing
		}
	}

	@Override
	public List<String> getMessages() {
		return em.createQuery("SELECT m.text FROM Message m", String.class)
				.getResultList();
	}
}
